var CACHE_INTERVAL_DAYS = 1;

function daysToMilliseconds(days) {
  if (days && Number(days)) {
    return Number(days) * 1000 * 60 * 60 * 24;
  }
}

function cacheIsStale() {
  var lastRefresh = Action.preferences.lastBunchRefresh;

  return (lastRefresh == undefined || Date.now() - lastRefresh > daysToMilliseconds(CACHE_INTERVAL_DAYS));
}

function getDefault (key) {
  var plistPath = LaunchBar.homeDirectory + '/Library/Preferences/com.brettterpstra.Bunch.plist'
  return LaunchBar.execute('/usr/bin/defaults', 'read', plistPath, key);
}

function refreshPrefs () {
  var pref = getDefault('configDir');
  LaunchBar.debugLog('cfgDir', pref);
  configDir = pref.replace(/^\s*(\S+.*?)[\s\n]*/,'$1').replace(/\/?$/,'/');

  Action.preferences.configDir = configDir;

  var defaultMethod = Action.preferences.defaultMethod;
  if (defaultMethod === undefined) {
    pref = getDefault('toggleBunches');
    defaultMethod = pref.trim() === '1' ? 'toggle' : 'open';
  }
  Action.preferences.defaultMethod = defaultMethod;

  Action.preferences.lastBunchRefresh = Date.now();
}

function configDir () {
  var configDir;

  configDir = Action.preferences.configDir;

  if (configDir === undefined || configDir === "/" || cacheIsStale()) {
    refreshPrefs();
    configDir = Action.preferences.configDir;
  }
  return configDir;
}

function getMethod() {
  var method,
      o = LaunchBar.options;

  if (o['controlKey']) {
    method = 'close';
  } else if (o['shiftKey']) {
    method = 'open';
  } else if (o['commandKey']) {
    method = 'toggle';
  } else {
    method = Action.preferences.defaultMethod;
    if (method === undefined || cacheIsStale()) {
      refreshPrefs();
      method = Action.preferences.defaultMethod;
    }
  }
  return method;
}

function capitalize(string) {
  return string.charAt(0).toUpperCase() + string.slice(1);
}

function runWithString(string) {
    var url = generateURL('raw', {'txt': string});
    return [{
     'title'    : `Run string as Bunch`,
     'url'      : url,
     'subtitle' : url,
     'icon'     : 'BunchDoc.png'
    }];
}

function runWithPaths(paths) {
  paths.forEach(function(path) {
    var url = generateURL('raw',{'file': path});
    LaunchBar.openURL(url);
  });

  return false;
}

function generateChildren (method, title) {
  var methods = ['open','close'],
      items = [];
  for (i in methods) {
    var _method = methods[i];
    if (_method !== method) {
      items.push({
        'title'    : capitalize(_method) + ' ' + title,
        'url'      : generateURL(_method, title),
        'icon'     : iconFor(_method)
       });
    }
  }

  return items;
}

function iconFor(method) {
  switch(method) {
    case 'open':
        return 'BunchDocOpen.png'
        break;
    case 'close':
        return 'BunchDocClose.png'
        break;
    case 'toggle':
        return 'BunchDocToggle.png'
        break;
    default:
        return 'BunchDoc.png'
  }
}

function run() {
  var data,
      items = [],
      cfgDir = configDir(),
      ls, files,
      title;

  var method = getMethod();

  var contents = File.getDirectoryContents(cfgDir);

  contents.forEach(function(item) {
    if (/\.bunch$/.test(item)) {
      title = item.replace(/(?:.*\/)?([^\/]+)\.bunch$/,'$1');
      var item = {
        'title'    : capitalize(method) + ' ' + title,
        'url'      : generateURL(method, title),
        'children' : generateChildren(method, title),
        'icon'     : iconFor(method)
        };

      items.push(item);
    }
  });

  var settings = [
      {
        'title'               : 'Refresh Preferences',
        'subtitle'            : `This action caches Bunch settings for ${CACHE_INTERVAL_DAYS} days, use this to force refresh`,
        'url'                 : 'x-launchbar:action/com.brettterpstra.launchbar.bunch/refresh',
        'icon'                : 'BunchDoc.png',
        'alwaysShowsSubtitle' : true
      },
      {
          'title'               : 'Set Default Bunch Action',
          'subtitle'            : `Default to open or toggle`,
          'icon'                : 'BunchDoc.png',
          'alwaysShowsSubtitle' : true,
          'children'            : [
            {
              'title'               : 'Open',
              'url'                 : 'x-launchbar:action/com.brettterpstra.launchbar.bunch/setPref?defaultMethod=open',
              'icon'                : 'BunchDocOpen.png'
            },
            {
              'title'               : 'Toggle',
              'url'                 : 'x-launchbar:action/com.brettterpstra.launchbar.bunch/setPref?defaultMethod=toggle',
              'icon'                : 'BunchDocToggle.png'
            }
          ]
      }
    ];

  return items.concat(settings);
}

function generateURL(method, parameters) {
  if (/(open|close|toggle)/.test(method) && typeof parameters === 'string') {
    return 'x-bunch://' + method + '?bunch=' + parameters;
  } else if (typeof parameters === 'object') {
    var queries = [],
        queryString = "?";
    for (var key in parameters) {
      queries.push(`${key}=${parameters[key]}`);
    }
    queryString += queries.join("&");
    return 'x-bunch://' + method + queryString;
  } else {
    return null;
  }
}
